@@ -36,7 +36,6 @@ BASE_PROMPT = """You have an instance `env` with methods:
 - `env.get_llm_response(text)` asks AI about a string `text`.
 - `env.retrieve_information(prompt, entire_page=False)` returns a string, information from a page given a prompt. Use prompt="Summarize:" for summaries. Uses all the text if entire_page=True and only visible text if False. Invoked with commands like "retrieve", "find in the page", or similar.
 - `env.ask_llm_to_find_element(description)` asks AI to find an GPTWebElement that matches the description. It returns None if it cannot find an element that matches the description, so you must check for that.
-- `env.query_memory(prompt)` asks AI with a prompt to query its memory (an embeddings index) of the web pages it has browsed.
 - `env.save(text, filename)` saves the string `text` to a file `filename`.
 - `env.get_text_from_page(entire_page)` returns the free text from the page. If entire_page is True, it returns all the text from HTML doc. If False, returns only visible text.
 
@@ -47,10 +46,10 @@ GPTWebElement has functions:
 4. `element.is_displayed()` returns if the element is visible.
 5. Do NOT use `element.send_keys(text)` or `element.click()`. Use `env.send_keys(text)` and `env.click(element)` instead.
 
-In xpaths, instead of using text(), use normalize-space() to get the text of an element.
-The xpath for an element whose text is or contains "text" is "//*[normalize-space() = 'text']". The xpath for "contains text" is "//*[contains(normalize-space(), 'text')]".
 The xpath of a text box is usually "//input|//div[@role = 'textarea']|//div[@role = 'textbox']".
+The xpath of text is usually "//*[string-length(text()) > 0]".
 The xpath for a button is usually "//button|//div[@role = 'button']", but it may sometimes also be an anchor.
+The xpath for an element whose text is "text" is "//*[normalize-space() = 'text']". The xpath for "contains text" is "//*[contains(normalize-space(), 'text')]".
 
 Your code must obey the following constraints:
 1. Respect case sensitivity in the instructions.
@@ -169,12 +168,15 @@ class InstructionCompiler:
     def _parse_instructions_into_queue(self, instructions) -> List:
         """Parse the instructions into a list of instructions."""
 
-        # First pass queue reads all of the functions and removes them
-        # from the string. Second pass queue is what collates the blocks
-        # that are fed into the LLM.
+        # First pass queue reads all of the functions and removes them.
+        # Second pass queue injects those functions for the INJECT_FUNCTION.
+        # Third pass queue is what collates the blocks that are fed into the
+        # LLM.
+        # Final queue is what is returned.
         self.functions = {}
         first_pass_queue = instructions.split("\n")
         second_pass_queue = []
+        third_pass_queue = []
         final_queue = []
 
         # First, parse all the functions, which are denoted by a line that
@@ -201,21 +203,10 @@ class InstructionCompiler:
             else:
                 second_pass_queue.append(line)
 
-        # Then parse the rest of the instructions. Every contiguous set of
-        # lines that do not start with "RUN FUNCTION" should be collated
-        # into a single block. For any line that starts with
-        # "RUN_FUNCTION name", then replace it with the respective function
-        # body from the dict.
+        # Second pass, inject the functions into the queue.
         while second_pass_queue:
             line = second_pass_queue.pop(0)
-            if not line:
-                continue
-
-            if line.startswith(RUN_FUNCTION_TOKEN):
-                function_name = line.split(" ")[-1]
-                function_body = self.functions[function_name]
-                final_queue.append(function_body)
-            elif line.startswith(INJECT_FUNCTION_TOKEN):
+            if line.startswith(INJECT_FUNCTION_TOKEN):
                 # Add in the instructions from the function cache.
                 # NOTE! The distinction between INJECT_FUNCTION and
                 # RUN_FUNCTION is that RUN_FUNCTION will add the function
@@ -226,16 +217,33 @@ class InstructionCompiler:
                 function_lines = [line for line in function_body.split("\n") if line]
                 function_lines.extend(second_pass_queue)
                 second_pass_queue = function_lines
+            else:
+                third_pass_queue.append(line)
+
+        # Then parse the rest of the instructions. Every contiguous set of
+        # lines that do not start with "RUN FUNCTION" should be collated
+        # into a single block. For any line that starts with
+        # "RUN_FUNCTION name", then replace it with the respective function
+        # body from the dict.
+        while third_pass_queue:
+            line = third_pass_queue.pop(0)
+            if not line:
+                continue
+
+            if line.startswith(RUN_FUNCTION_TOKEN):
+                function_name = line.split(" ")[-1]
+                function_body = self.functions[function_name]
+                final_queue.append(function_body)
             else:
                 # Otherwise, just add all contiguous lines that do not start with
                 # RUN_FUNCTION.
                 instruction_block = line + "\n"
-                while second_pass_queue:
-                    line = second_pass_queue.pop(0)
+                while third_pass_queue:
+                    line = third_pass_queue.pop(0)
                     if line.startswith(RUN_FUNCTION_TOKEN):
                         # Add it back to the queue and stop constructing this
                         # block.
-                        second_pass_queue.insert(0, line)
+                        third_pass_queue.insert(0, line)
                         break
                     else:
                         instruction_block += line + "\n"
