@@ -19,7 +19,6 @@ from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.support.relative_locator import locate_with
 from .compilers.instruction_compiler import InstructionCompiler
-from .memories import Memory
 
 TIME_BETWEEN_ACTIONS = 0.1
 
@@ -50,12 +49,11 @@ class GPTSeleniumAgent:
         instructions,
         chromedriver_path,
         chrome_options={},
+        model_for_instructions="gpt-3.5-turbo",
+        model_for_responses="gpt-3.5-turbo",
         user_data_dir="user_data",
         headless=False,
         retry=False,
-        model_for_instructions="gpt-3.5-turbo",
-        model_for_responses="gpt-3.5-turbo",
-        enable_memory=False,
         debug=False,
         debug_html_folder="",
         instruction_output_file=None,
@@ -95,7 +93,6 @@ class GPTSeleniumAgent:
         self.should_retry = retry
         self.debug = debug
         self.debug_html_folder = debug_html_folder
-        self.enable_memory = enable_memory
 
         """Fire up the compiler."""
         self.instruction_compiler = InstructionCompiler(
@@ -103,12 +100,6 @@ class GPTSeleniumAgent:
             model=self.model_for_instructions,
         )
 
-        """Set up the memory."""
-        self.memory = None
-        if self.enable_memory:
-            logger.info("Enabling memory.")
-            self.memory = Memory()
-
         """Set up the driver."""
         _chrome_options = webdriver.ChromeOptions()
         _chrome_options.add_argument(f"user-data-dir={user_data_dir}")
@@ -371,11 +362,7 @@ class GPTSeleniumAgent:
         if not url.startswith("http"):
             url = "http://" + url
         self.driver.get(url)
-        time.sleep(2)
-        if self.enable_memory:
-            # Get all the visible text from the page and add it to the memory.
-            text = self.get_text_from_page(entire_page=False)
-            self.memory.add(text)
+        time.sleep(3)
 
     def scroll(self, direction=None, iframe=None):
         assert direction in ["up", "down", "left", "right"]
@@ -536,12 +523,6 @@ class GPTSeleniumAgent:
 
         return self.instruction_compiler.get_completion(prompt, temperature, model)
 
-    def query_memory(self, prompt):
-        """Queries the memory of the LLM."""
-        if self.enable_memory:
-            return self.memory.ask(prompt)
-        logger.error("Memory is disabled.")
-
     def ask_llm_to_find_element(self, element_description):
         """Clean the HTML from self.driver, ask GPT-Index to find the element,
         and return Selenium code to access it. Return a GPTWebElement."""
